@@ -7,8 +7,6 @@
 #
 # Contact: Kyle Lahnakoski (kyle@lahnakoski.com)
 #
-from __future__ import absolute_import, division, unicode_literals
-
 import base64
 import io
 import os
@@ -18,10 +16,10 @@ from datetime import datetime
 from mimetypes import MimeTypes
 from tempfile import NamedTemporaryFile, mkdtemp
 
-from mo_dots import Null, coalesce, get_module, is_list
+from mo_dots import Null, coalesce, get_module, is_list, to_data
 from mo_files import mimetype
 from mo_files.url import URL
-from mo_future import text, is_text
+from mo_future import text, is_text, ConfigParser, StringIO
 from mo_logs import Except, Log
 from mo_logs.exceptions import get_stacktrace
 from mo_math import randoms
@@ -188,7 +186,7 @@ class File(object):
         """
         RETURN NEW FILE WITH EXTENSION ADDED (OLD EXTENSION IS A SUFFIX)
         """
-        return File(self._filename + "." + text(ext))
+        return File(self._filename + "." + str(ext))
 
     def set_name(self, name):
         """
@@ -258,9 +256,7 @@ class File(object):
                 else:
                     return f.read()
         except Exception as e:
-            Log.error(
-                "Problem reading file {{filename}}", filename=self.abs_path, cause=e
-            )
+            Log.error("Problem reading file {{filename}}", filename=self.abs_path, cause=e)
 
     def write_bytes(self, content):
         if not self.parent.exists:
@@ -280,10 +276,7 @@ class File(object):
             self.parent.create()
         with open(self._filename, "wb") as f:
             if is_list(content) and self.key:
-                Log.error(
-                    "list of data and keys are not supported, encrypt before sending to"
-                    " file"
-                )
+                Log.error("list of data and keys are not supported, encrypt before sending to file")
 
             if is_list(content):
                 pass
@@ -302,6 +295,22 @@ class File(object):
                 else:
                     f.write(d.encode("utf8"))
 
+    def read_ini(self, encoding="utf8"):
+        buff = StringIO(self.read(encoding))
+        config = ConfigParser()
+        config._read(buff, "dummy")
+
+        output = {}
+        for section in config.sections():
+            output[section] = s = {}
+            for k, v in config.items(section):
+                s[k] = v
+        return to_data(output)
+
+    def write_ini(self, data, encoding="utf8"):
+        with open(self.os_path, 'w') as configfile:
+            ConfigParser().write(configfile)
+
     def __iter__(self):
         # NOT SURE HOW TO MAXIMIZE FILE READ SPEED
         # http://stackoverflow.com/questions/8009882/how-to-read-large-file-line-by-line-in-python
@@ -318,9 +327,7 @@ class File(object):
                         yield line.decode("utf8").rstrip()
             except Exception as e:
                 Log.error(
-                    "Can not read line from {{filename}}",
-                    filename=self._filename,
-                    cause=e,
+                    "Can not read line from {{filename}}", filename=self._filename, cause=e,
                 )
 
         return output()
@@ -364,29 +371,24 @@ class File(object):
             elif os.path.isfile(self._filename):
                 os.remove(self._filename)
             return self
-        except Exception as e:
-            e = Except.wrap(e)
-            if "The system cannot find the path specified" in e:
+        except Exception as cause:
+            cause = Except.wrap(cause)
+            if (
+                "The system cannot find the path specified" in cause
+                or "The system cannot find the file specified" in cause
+            ):
                 return
-            Log.error("Could not remove file", e)
+            Log.error("Could not remove file", cause)
 
     def backup(self):
         path = self._filename.split("/")
         names = path[-1].split(".")
         if len(names) == 1 or names[0] == "":
-            backup = File(
-                self._filename
-                + ".backup "
-                + datetime.utcnow().strftime("%Y%m%d %H%M%S")
-            )
+            backup = File(self._filename + ".backup " + datetime.utcnow().strftime("%Y%m%d %H%M%S"))
         else:
             backup = File.new_instance(
                 "/".join(path[:-1]),
-                ".".join(names[:-1])
-                + ".backup "
-                + datetime.now().strftime("%Y%m%d %H%M%S")
-                + "."
-                + names[-1],
+                ".".join(names[:-1]) + ".backup " + datetime.now().strftime("%Y%m%d %H%M%S") + "." + names[-1],
             )
         File.copy(self, backup)
         return backup
@@ -398,9 +400,7 @@ class File(object):
             pass
         except Exception as e:
             Log.error(
-                "Could not make directory {{dir_name}}",
-                dir_name=self._filename,
-                cause=e,
+                "Could not make directory {{dir_name}}", dir_name=self._filename, cause=e,
             )
 
     @property
@@ -474,9 +474,6 @@ class File(object):
     def __data__(self):
         return self._filename
 
-    def __unicode__(self):
-        return self.abs_path
-
     def __eq__(self, other):
         return isinstance(other, File) and other.abs_path == self.abs_path
 
@@ -505,12 +502,7 @@ class TempDirectory(File):
     def __exit__(self, exc_type, exc_val, exc_tb):
         from mo_threads import Thread
 
-        Thread.run(
-            "delete dir " + self.stem,
-            delete_daemon,
-            file=self,
-            caller_stack=get_stacktrace(1),
-        ).release()
+        Thread.run("delete dir " + self.stem, delete_daemon, file=self, caller_stack=get_stacktrace(1)).release()
 
 
 class TempFile(File):
@@ -535,12 +527,7 @@ class TempFile(File):
     def __exit__(self, exc_type, exc_val, exc_tb):
         from mo_threads import Thread
 
-        Thread.run(
-            "delete file " + self.rel_path,
-            delete_daemon,
-            file=self,
-            caller_stack=get_stacktrace(1),
-        ).release()
+        Thread.run("delete file " + self.rel_path, delete_daemon, file=self, caller_stack=get_stacktrace(1),).release()
 
 
 def _copy(from_, to_):
@@ -563,12 +550,10 @@ def datetime2string(value, format="%Y-%m-%d %H:%M:%S"):
         return value.strftime(format)
     except Exception as e:
         Log.error(
-            "Can not format {{value}} with {{format}}",
-            value=value,
-            format=format,
-            cause=e,
+            "Can not format {{value}} with {{format}}", value=value, format=format, cause=e,
         )
 
+
 def join_path(*path):
     def scrub(i, p):
         p = p.replace(os.sep, "/")
@@ -637,9 +622,7 @@ def delete_daemon(file, caller_stack, please_stop):
             e = Except.wrap(e)
             e.trace = e.trace[0:2] + caller_stack
             if num_attempts:
-                Log.warning(
-                    "problem deleting file {{file}}", file=file.abs_path, cause=e
-                )
+                Log.warning("problem deleting file {{file}}", file=file.abs_path, cause=e)
             (Till(seconds=10) | please_stop).wait()
         num_attempts += 1
 
@@ -651,6 +634,6 @@ def add_suffix(filename, suffix):
     path = filename.split("/")
     parts = path[-1].split(".")
     i = max(len(parts) - 2, 0)
-    parts[i] = parts[i] + "." + text(suffix).strip(".")
+    parts[i] = parts[i] + "." + str(suffix).strip(".")
     path[-1] = ".".join(parts)
     return File("/".join(path))
