@@ -6,7 +6,6 @@
 #
 # Contact: Kyle Lahnakoski (kyle@lahnakoski.com)
 #
-
 from mo_dots import (
     Data,
     Null,
@@ -18,7 +17,7 @@ from mo_dots import (
     unwraplist,
     is_null,
 )
-from mo_future import PY2, is_text, text, unichr, urlparse, is_binary
+from mo_future import is_text, text, unichr, urlparse, is_binary
 from mo_logs import Log
 
 
@@ -67,32 +66,34 @@ class URL(object):
             Log.error(u"problem parsing {{value}} to URL", value=value, cause=e)
 
     def __nonzero__(self):
-        if (
-            self.scheme
-            or self.host
-            or self.port
-            or self.path
-            or self.query
-            or self.fragment
-        ):
+        if self.scheme or self.host or self.port or self.path or self.query or self.fragment:
             return True
         return False
 
     def __bool__(self):
-        if (
-            self.scheme
-            or self.host
-            or self.port
-            or self.path
-            or self.query
-            or self.fragment
-        ):
+        if self.scheme or self.host or self.port or self.path or self.query or self.fragment:
             return True
         return False
 
     def __truediv__(self, other):
+        if self.scheme == "file" and not self.path:
+            # keep relative path
+            path = str(other).lstrip('/')
+        else:
+            path = f"{self.path.rstrip('/')}/{str(other).lstrip('/')}"
+
+        parts = []
+        for step in path.split("/"):
+            if step == ".":  # IGNORE
+                pass
+            elif step == ".." and parts and (len(parts) > 1 or parts[0]):  # BACK UP
+                parts.pop()
+            else:
+                parts.append(step)
+        path = "/".join(parts)
+
         output = self.__copy__()
-        output.path = output.path.rstrip("/") + "/" + text(other).lstrip("/")
+        output.path = path
         return output
 
     def __add__(self, other):
@@ -102,9 +103,6 @@ class URL(object):
         output.query += other
         return output
 
-    def __unicode__(self):
-        return self.__str__().decode("utf8")  # ASSUME chr<128 ARE VALID UNICODE
-
     def __copy__(self):
         output = URL(None)
         output.scheme = self.scheme
@@ -116,7 +114,7 @@ class URL(object):
         return output
 
     def decode(self, encoding=""):
-        return text(self).decode(encoding)
+        return str(self).decode(encoding)
 
     def __data__(self):
         return str(self)
@@ -125,7 +123,7 @@ class URL(object):
         return hash(str(self))
 
     def __eq__(self, other):
-        return text(self) == text(other)
+        return str(self) == str(other)
 
     def __str__(self):
         url = ""
@@ -136,11 +134,12 @@ class URL(object):
         if self.port:
             url = url + ":" + str(self.port)
         if self.path:
-            if self.path[0] == text("/"):
-                url += str(self.path)
+            path = str(self.path)
+            if self.host and path[0] != "/":
+                url += "/" + path
             else:
-                url += "/" + str(self.path)
-        if self.query:
+                url += path
+        if len(self.query):
             url = url + "?" + value2url_param(self.query)
         if self.fragment:
             url = url + "#" + value2url_param(self.fragment)
@@ -158,20 +157,12 @@ def hex2chr(hex):
         raise e
 
 
-if PY2:
-    _map2url = {chr(i): chr(i) for i in range(32, 128)}
-    for c in "{}<>;/?@&=+$%,+":
-        _map2url[c] = "%" + str(int2hex(ord(c), 2))
-    for i in range(128, 256):
-        _map2url[chr(i)] = "%" + str(int2hex(i, 2))
-    _map2url[chr(32)] = "+"
-else:
-    _map2url = {i: unichr(i) for i in range(32, 128)}
-    for c in b"{}<>;/?@&=+$%,+":
-        _map2url[c] = "%" + int2hex(c, 2)
-    for i in range(128, 256):
-        _map2url[i] = "%" + str(int2hex(i, 2))
-    _map2url[32] = "+"
+_map2url = {i: unichr(i) for i in range(32, 128)}
+for c in b"{}<>;/?@&=+$%,+":
+    _map2url[c] = "%" + int2hex(c, 2)
+for i in range(128, 256):
+    _map2url[i] = "%" + str(int2hex(i, 2))
+_map2url[32] = "+"
 
 
 names = ["path", "query", "fragment"]
@@ -324,9 +315,7 @@ def from_paths(value):
         if any(not p.endswith("]") for p in path[1:]):
             Log.error("expecting square brackets to be paired")
         path = [
-            int(pp) if is_integer(pp) else pp
-            for i, p in enumerate(path)
-            for pp in [p.rstrip("]") if i > 0 else p]
+            int(pp) if is_integer(pp) else pp for i, p in enumerate(path) for pp in [p.rstrip("]") if i > 0 else p]
         ]
 
         d = output
@@ -338,9 +327,7 @@ def from_paths(value):
                     d[p] = []
             elif is_text(q) == is_list(d[p]):
                 Log.error(
-                    "can not index {{type}} with {{key}}",
-                    type=type(d[p]).__name__,
-                    key=q,
+                    "can not index {{type}} with {{key}}", type=type(d[p]).__name__, key=q,
                 )
 
             d = d[p]
@@ -383,12 +370,7 @@ def value2url_param(value):
         if any(is_data(v) or is_many(v) for v in value):
             output = _encode(value2json(value))
         else:
-            output = ",".join(
-                vv
-                for v in value
-                for vv in [value2url_param(v)]
-                if vv or vv == 0
-            )
+            output = ",".join(vv for v in value for vv in [value2url_param(v)] if vv or vv == 0)
     else:
         output = _encode(value2json(value))
     return output
